@@ -3,9 +3,11 @@ from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
 from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
 from django.contrib.contenttypes.models import ContentType
-from comment.models import Comment
+
+from comment.models import Comment, ReactionInstance, FlagInstance
 from comment.forms import CommentForm
 from comment.utils import has_valid_profile
+from comment.managers import FlagInstanceManager
 
 register = template.Library()
 
@@ -95,6 +97,7 @@ def render_comments(obj, request, oauth=False, comments_per_page=10):
     if not login_url.startswith('/'):
         login_url = '/' + login_url
 
+    allowed_flags = getattr(settings, 'COMMENT_FLAGS_ALLOWED', 0)
     context = {
         'comment_form': CommentForm(),
         'model_object': obj,
@@ -103,12 +106,13 @@ def render_comments(obj, request, oauth=False, comments_per_page=10):
         'oauth': oauth,
         'login_url': login_url,
         'has_valid_profile': has_valid_profile(),
+        'allowed_flags': allowed_flags,
         'comments_per_page': comments_per_page
     }
     return context
 
 
-register.inclusion_tag('comment/base.html')(render_comments)
+register.inclusion_tag('comment/comments/base.html')(render_comments)
 
 
 def include_static():
@@ -139,3 +143,44 @@ register.inclusion_tag('comment/bootstrap.html')(include_bootstrap)
 def render_field(field, **kwargs):
     field.field.widget.attrs.update(kwargs)
     return field
+
+
+@register.simple_tag(name='has_reacted')
+def has_reacted(comment, user, reaction):
+    """
+    Returns whether a user has reacted with a particular reaction on a comment or not.
+
+    Args:
+        comment (Comment): comment to be queried about.
+        user (User): user to be queried about.
+        reaction (str): reaction to be queried about.
+
+    Returns:
+        bool
+    """
+    if user.is_authenticated:
+        reaction_type = getattr(ReactionInstance.ReactionType, reaction.upper(), None)
+        if not reaction_type:
+            raise template.TemplateSyntaxError(
+                'Reaction must be an valid ReactionManager.RelationType. {} is not'.format(reaction)
+                )
+        return ReactionInstance.objects.filter(
+            user=user,
+            reaction_type=reaction_type.value,
+            reaction__comment=comment
+            ).exists()
+
+    return False
+
+
+@register.filter(name='has_flagged')
+def has_flagged(user, comment):
+    if user.is_authenticated:
+        return FlagInstance.objects.filter(user=user, flag__comment=comment).exists()
+
+    return False
+
+
+@register.simple_tag(name='render_flag_reasons')
+def render_flag_reasons():
+    return FlagInstanceManager.reasons_list
