@@ -4,8 +4,9 @@ from django.conf import settings
 from django.test import RequestFactory
 
 from comment.models import Comment
-from comment.api.serializers import get_profile_model, get_user_fields, UserSerializer, CommentCreateSerializer
-from comment.api.permissions import IsOwnerOrReadOnly, ContentTypePermission, ParentIdPermission
+from comment.api.serializers import get_profile_model, get_user_fields, UserSerializer, CommentCreateSerializer, \
+    CommentSerializer
+from comment.api.permissions import IsOwnerOrReadOnly, ContentTypePermission, ParentIdPermission, FlagEnabledPermission
 from comment.api.views import CommentList
 from comment.tests.base import BaseCommentTest
 
@@ -17,11 +18,14 @@ class APIBaseTest(BaseCommentTest):
         self.comment_2 = self.create_comment(self.content_object_1)
         self.comment_3 = self.create_comment(self.content_object_1)
         self.comment_4 = self.create_comment(self.content_object_1, parent=self.comment_1)
+        self.reaction_1 = self.create_reaction_instance(self.user_1, self.comment_1, 'like')
 
         self.comment_5 = self.create_comment(self.content_object_2)
         self.comment_6 = self.create_comment(self.content_object_2)
         self.comment_7 = self.create_comment(self.content_object_2, parent=self.comment_5)
         self.comment_8 = self.create_comment(self.content_object_2, parent=self.comment_5)
+        self.reaction_2 = self.create_reaction_instance(self.user_1, self.comment_5, 'dislike')
+
         self.addCleanup(patch.stopall)
 
 
@@ -31,6 +35,7 @@ class APIPermissionTest(APIBaseTest):
         self.owner_permission = IsOwnerOrReadOnly()
         self.content_type_permission = ContentTypePermission()
         self.parent_permission = ParentIdPermission()
+        self.flag_enabled_permission = FlagEnabledPermission()
         self.factory = RequestFactory()
         self.view = CommentList()
 
@@ -115,6 +120,13 @@ class APIPermissionTest(APIBaseTest):
         request = self.factory.get('/api/comments/create/?type=post&id=2&parent_id=0')
         self.assertTrue(self.parent_permission.has_permission(request, self.view))
 
+    def test_flag_enabled_permission(self):
+        request = self.factory.get('/')
+        settings.COMMENT_FLAGS_ALLOWED = 0
+        self.assertFalse(self.flag_enabled_permission.has_permission(request, self.view))
+        settings.COMMENT_FLAGS_ALLOWED = 1
+        self.assertTrue(self.flag_enabled_permission.has_permission(request, self.view))
+
 
 class APICommentViewTest(APIBaseTest):
     def test_can_retrieve_comments_list(self):
@@ -230,6 +242,121 @@ class APICommentViewTest(APIBaseTest):
         self.assertEqual(response.status_code, 204)
         self.assertEqual(Comment.objects.all().count(), count - 2)
 
+    def test_react_to_comment_success(self):
+        # post like - comment has no reaction
+        self.assertEqual(self.comment_3.likes, 0)
+        self.assertEqual(self.comment_3.dislikes, 0)
+        response = self.client.post(f'/api/comments/{self.comment_3.id}/react/like/')
+        self.assertEqual(response.status_code, 200)
+        self.comment_3.reaction.refresh_from_db()
+        self.assertEqual(self.comment_3.likes, 1)
+        self.assertEqual(self.comment_3.dislikes, 0)
+
+        # post dislike - comment is liked by the user
+        response = self.client.post(f'/api/comments/{self.comment_3.id}/react/dislike/')
+        self.assertEqual(response.status_code, 200)
+        self.comment_3.reaction.refresh_from_db()
+        self.assertEqual(self.comment_3.likes, 0)
+        self.assertEqual(self.comment_3.dislikes, 1)
+
+        # post dislike - comment is disliked by the user => comment reaction is removed
+        response = self.client.post(f'/api/comments/{self.comment_3.id}/react/dislike/')
+        self.assertEqual(response.status_code, 200)
+        self.comment_3.reaction.refresh_from_db()
+        self.assertEqual(self.comment_3.likes, 0)
+        self.assertEqual(self.comment_3.dislikes, 0)
+
+    def test_react_to_comment_with_invalid_reaction_type(self):
+        response = self.client.post(f'/api/comments/{self.comment_3.id}/react/invalid_type/')
+        self.assertEqual(response.status_code, 400)
+
+
+class APICommentFlagViewTest(APIBaseTest):
+
+    def get_url(self, c_id=None):
+        if not c_id:
+            c_id = self.comment.id
+
+        return f'/api/comments/{c_id}/flag/'
+
+    def setUp(self):
+        super().setUp()
+        self.comment = self.comment_1
+        self.user = self.user_1
+        self.flag = self.create_flag_instance(self.user_2, self.comment_2)
+        self.flag_data = {
+            'reason': 1,
+            'info': '',
+        }
+
+    def test_flag_to_comment(self):
+        comment = self.comment
+        data = self.flag_data
+        # flag - comment has no flags
+        url = self.get_url()
+        self.assertEqual(comment.flag.count, 0)
+        response = self.client.post(url, data=data)
+        self.assertEqual(response.status_code, 200)
+        self.assertFalse(response.json()['is_flagged'])
+        comment.flag.refresh_from_db()
+        self.assertEqual(comment.flag.count, 1)
+
+    def test_flag_comment_when_flagging_not_enabled(self):
+        settings.COMMENT_FLAGS_ALLOWED = 0
+        comment = self.comment
+        data = self.flag_data
+        url = self.get_url()
+        self.assertEqual(comment.flag.count, 0)
+        response = self.client.post(url, data=data)
+        self.assertEqual(response.status_code, 403)
+        settings.COMMENT_FLAGS_ALLOWED = 1
+
+    def test_unflag_to_comment(self):
+        comment = self.comment_2
+        user = self.user_2
+        self.client.force_login(user)
+        url = self.get_url(comment.id)
+        # un-flag - comment is flagged by the user and no reason is passed
+        response = self.client.post(url)
+        self.assertEqual(response.status_code, 200)
+        comment.flag.refresh_from_db()
+        self.assertEqual(comment.flag.count, 0)
+
+    def test_flag_to_previous_comments(self):
+        """Maintains backward compatibility"""
+        comment = self.comment
+        url = self.get_url()
+        data = self.flag_data
+        comment.flag.delete()   # delete the flag object
+        response = self.client.post(url, data=data)
+        self.assertEqual(response.status_code, 200)
+        comment.refresh_from_db()
+        self.assertEqual(comment.flag.count, 1)
+
+    def test_is_flagged_value(self):
+        # test is_flagged property
+        comment = self.create_comment(self.content_object_1)
+        data = self.flag_data
+        url = self.get_url(comment.id)
+        settings.COMMENT_FLAGS_ALLOWED = 1
+        response = self.client.post(url, data=data)
+        self.assertEqual(response.status_code, 200)
+        self.assertFalse(response.json()['is_flagged'])
+        comment.flag.refresh_from_db()
+        self.assertEqual(comment.flag.count, 1)
+        self.client.force_login(self.user_2)
+        response = self.client.post(url, data=data)
+        self.assertEqual(response.status_code, 200)
+        self.assertTrue(response.json()['is_flagged'])
+        comment.flag.refresh_from_db()
+        self.assertEqual(comment.flag.count, 2)
+
+    def test_flag_to_comment_with_invalid_reason(self):
+        data = self.flag_data
+        data.update({'reason': -1})
+        response = self.client.post(self.get_url(), data=data)
+        self.assertEqual(response.status_code, 400)
+
 
 class APICommentSerializers(APIBaseTest):
     def test_get_profile_model(self):
@@ -249,7 +376,7 @@ class APICommentSerializers(APIBaseTest):
         self.assertIsNone(profile)
 
         # success
-        setattr(settings, 'PROFILE_APP_NAME', 'profile')
+        setattr(settings, 'PROFILE_APP_NAME', 'user_profile')
         profile = get_profile_model()
         self.assertIsNotNone(profile)
 
@@ -338,3 +465,39 @@ class APICommentSerializers(APIBaseTest):
         reply_count = serializer.get_reply_count(self.comment_4)
         self.assertIsNone(replies)
         self.assertIsNone(reply_count)
+
+        # test reaction fields
+        likes = serializer.get_likes(self.comment_2)
+        dislikes = serializer.get_dislikes(self.comment_2)
+        self.assertIsNotNone(likes)
+        self.assertIsNotNone(dislikes)
+
+        # test default reaction count
+        self.assertEqual(likes, 0)
+        self.assertEqual(dislikes, 0)
+
+        # get reaction count
+        self.assertEqual(serializer.get_likes(self.comment_1), 1)
+        self.assertEqual(serializer.get_dislikes(self.comment_5), 1)
+
+        # test is_flagged field
+        is_flagged = serializer.get_is_flagged(self.comment_2)
+        self.assertIsNotNone(is_flagged)
+        # test default value
+        self.assertEqual(False, is_flagged)
+
+        mocked_hasattr = patch('comment.api.serializers.hasattr').start()
+        mocked_hasattr.return_value = False
+        self.assertIsNone(serializer.get_likes(self.comment_2))
+        self.assertIsNone(serializer.get_dislikes(self.comment_2))
+        self.assertEqual(False, serializer.get_is_flagged(self.comment_2))
+
+    def test_passing_context_to_serializer(self):
+        serializer = CommentSerializer(self.comment_1)
+        self.assertFalse(serializer.fields['content'].read_only)
+
+        serializer = CommentSerializer(self.comment_1, context={'reaction_update': True})
+        self.assertTrue(serializer.fields['content'].read_only)
+
+        serializer = CommentSerializer(self.comment_1, context={'flag_update': True})
+        self.assertTrue(serializer.fields['content'].read_only)
